unit wa_dlg;

{ This is the original wa_dlg.h file from the new Winamp v2.9x API.
  Saivert has translated/adapted it for use with the Borland Delphi
  development environment (which uses the Object Pascal language).
  All comments added by me are marked with "[Saivert]".

  My homepage URL is:
    http://members.tripod.com/files_saivert/
  My e-mail address is (I got spam protection, so don't bother...):
    saivert@email.com

  Following is the original Nullsoft license: }


(*
** Copyright (C) 2003 Nullsoft, Inc.
**
** This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held 
** liable for any damages arising from the use of this software. 
**
** Permission is granted to anyone to use this software for any purpose, including commercial applications, and to 
** alter it and redistribute it freely, subject to the following restrictions:
**
**   1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. 
**      If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
**
**   2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
**
**   3. This notice may not be removed or altered from any source distribution.
**
*)

interface

uses Windows, Messages, wa_ipc;

(*
  
  dont know where to put this yet :)
       1) gen.bmp has a generic window frame for plugins to use.
          its format is similar to the minibrowser's.
          In addition gen.bmp includes a font for the titlebar, in both
          highlight and no-highlight modes. The font is variable width,
          and it uses the first color before the letter A as the delimiter.
          The no-highlight form of letter must be the same width as the
          highlight form.
       2) genex.bmp has button and scrollbar images, as well as some individual
          pixels that describe the colors for the dialog. The button and
          scrollbar images should be self explanatory (note that the buttons
          have 4 pixel sized edges that are not stretched, and the center is
          stretched), and the scrollbars do something similar.
          The colors start at (48,0) and run every other pixel. The meaning
          of each pixel is:
            x=48: item background (background to edits, listviews etc)
            x=50: item foreground (text color of edit/listview, etc)
            x=52: window background (used to set the bg color for the dialog)
            x=54: button text color
            x=56: window text color
            x=58: color of dividers and sunken borders
            x=60: selection color for listviews/playlists/etc
            x=62: listview header background color
            x=64: listview header text color
            x=66: listview header frame top color
            x=68: listview header frame middle color
            x=70: listview header frame bottom color
            x=72: listview header empty color
            x=74: scrollbar foreground color
            x=76: scrollbar background color
            x=78: inverse scrollbar foreground color
            x=80: inverse scrollbar background color
            x=82: scrollbar dead area color
*)

const
  DCW_SUNKENBORDER = $00010000;
  DCW_DIVIDER = $00020000;

type
  TWADlgCol =(
  WADLG_ITEMBG,
  WADLG_ITEMFG,
  WADLG_WNDBG,
  WADLG_BUTTONFG,
  WADLG_WNDFG,
  WADLG_HILITE,
  WADLG_SELCOLOR,
  WADLG_LISTHEADER_BGCOLOR,
  WADLG_LISTHEADER_FONTCOLOR,
  WADLG_LISTHEADER_FRAME_TOPCOLOR,
  WADLG_LISTHEADER_FRAME_MIDDLECOLOR,
  WADLG_LISTHEADER_FRAME_BOTTOMCOLOR,
  WADLG_LISTHEADER_EMPTY_BGCOLOR,
  WADLG_SCROLLBAR_FGCOLOR,
  WADLG_SCROLLBAR_BGCOLOR,
  WADLG_SCROLLBAR_INV_FGCOLOR,
  WADLG_SCROLLBAR_INV_BGCOLOR,
  WADLG_SCROLLBAR_DEADAREA_COLOR,
  WADLG_NUM_COLORS
  );


procedure WADlg_init(hwndWinamp: HWND);
procedure WADlg_close;

function WADlg_getColor(idx: TWADlgCol): Integer;

function WADlg_handleDialogMsgs(hwndDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): Integer;
{ int WADlg_handleDialogMsgs(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam); }

procedure WADlg_DrawChildWindowBorders(hwndDlg: HWND; tab: PInteger; tabsize: Integer);
{ each entry in tab would be the id | DCW_* }
{ void WADlg_DrawChildWindowBorders(HWND hwndDlg, int *tab, int tabsize); }

function WADlg_getBitmap: HBITMAP;


{ define WA_DLG_IMPLEMENT in one of your source files before including this .h
  if you are making a media library plugin, you dont need to do this, look at view_ex for
  an example of how to get the function *'s via an IPC message. }

{$DEFINE WA_DLG_IMPLEMENT}
{$IF Defined(WA_DLG_IMPLEMENT)}

var
  wadlg_lastbrush: HBRUSH;
  wadlg_bitmap: HBITMAP;
  wadlg_colors: array[0..Integer(WADLG_SCROLLBAR_DEADAREA_COLOR)] of Integer;
  wadlg_defcolors: array[0..Integer(WADLG_SCROLLBAR_DEADAREA_COLOR)] of Integer;

(* [Saivert] original C statements

static HBRUSH wadlg_lastbrush;
static HBITMAP wadlg_bitmap; // load this manually
static int wadlg_colors[WADLG_NUM_COLORS];
static int wadlg_defcolors[WADLG_NUM_COLORS]=
{
RGB(0,0,0),
RGB(0,255,0),
RGB(36,36,60),
RGB(57,56,66),
RGB(255,255,255),
RGB(132,148,165),
RGB(0,0,198),
RGB(36*2,36*2,60*2),
RGB(255,255,255),
RGB(36*3,36*3,60*3),
RGB(36,36,60),
RGB(36*0.5,36*0.5,60*0.5),
RGB(36,36,60),
RGB(36*1,36*1,60*1),
RGB(36*1,36*1,60*1),
RGB(121,130,150),
RGB(78,88,110),
RGB(36*1,36*1,60*1),
};

*)

{$IFEND}

implementation

{$IF Defined(WA_DLG_IMPLEMENT)}

function WADlg_getColor(idx: TWADlgCol): Integer;
begin
  if (idx < Low(TWADlgCol)) or (idx >= High(TWADlgCol)) then
    result := 0
  else result := wadlg_colors[Ord(idx)];
end;

function WADlg_getBitmap: HBITMAP;
begin
  result := wadlg_bitmap;
end;

procedure WADlg_init(hwndWinamp: HWND); { call this on init, or on WM_DISPLAYCHANGE }
var
  tmpDC: HDC;
  o: HGDIOBJ;
  x: Integer;
  a: Cardinal;
begin
  if wadlg_bitmap > 0 then DeleteObject(wadlg_bitmap);
  wadlg_bitmap := HBITMAP(SendMessage(hwndWinamp,WM_WA_IPC,0,IPC_GET_GENSKINBITMAP));
  if wadlg_bitmap > 0 then
  begin
    tmpDC := CreateCompatibleDC(0);
    o := SelectObject(tmpDC,HGDIOBJ(wadlg_bitmap));

    { [Saivert] original C code: "for (x = 0; x < WADLG_NUM_COLORS; x ++)" }

    for x := 0 to Ord(WADLG_NUM_COLORS) do
    begin
      (* [Saivert] original C code
      int a=GetPixel(tmpDC,48+x*2,0);
      if (a == CLR_INVALID || a == RGB(0,198,255)) a=wadlg_defcolors[x];
      wadlg_colors[x]=a;
      *)

      a := GetPixel(tmpDC,48+x*2,0);
      if (a = CLR_INVALID) or (a = RGB(0,198,255)) then a := wadlg_defcolors[x];
      wadlg_colors[x] := a;
    end;

    SelectObject(tmpDC,o);
    DeleteDC(tmpDC);
  end;
end;

procedure WADlg_close;
begin
  if wadlg_bitmap > 0 then DeleteObject(wadlg_bitmap);
  wadlg_bitmap := 0;
  if wadlg_lastbrush > 0 then DeleteObject(wadlg_lastbrush);
  wadlg_lastbrush := 0;
end;

procedure WADlg_dotLine(_hdc: HDC; left, top, len: Integer; vert: Boolean);
var
  i: Integer;
begin
  i := top and 1;
  while i < len-1 do
  begin
    if vert then
    begin
      MoveToEx(_hdc,left,top+i,nil);
      LineTo(_hdc,left,top+i+1);
    end else
    begin
      MoveToEx(_hdc,left+i,top,nil);
      LineTo(_hdc,left+i+1,top);
    end;

    Inc(i, 2);
  end;

(* [Saivert] original C code
  for(int i=(top&1);i<len-1;i+=2)
  {
    if(vert) {
      MoveToEx(hdc,left,top+i,NULL);
      LineTo(hdc,left,top+i+1);
    } else {
      MoveToEx(hdc,left+i,top,NULL);
      LineTo(hdc,left+i+1,top);
    }
  }
*)
end;

function WADlg_handleDialogMsgs(hwndDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): Integer;
var
  di: DRAWITEMSTRUCT;
  wt: array[0..255] of Char;
  r: TRect;
  _hdc: HDC;
  yoffs: Integer;
  pen: HPEN;
  bgcolor: Integer;
  lb: LOGBRUSH;
begin
	if uMsg = WM_DRAWITEM then
  begin
    di := PDrawItemStruct(lParam)^;

		if (di.CtlType = ODT_BUTTON) then
    begin
			GetDlgItemText(hwndDlg,wParam,wt,sizeof(wt));

      _hdc := CreateCompatibleDC(di.hDC);
      SelectObject(_hdc,wadlg_bitmap);

			r := di.rcItem;
      SetStretchBltMode(di.hDC,COLORONCOLOR);


      if (di.itemState and ODS_SELECTED) > 0 then
        yoffs := 15 else yoffs := 0;


      BitBlt(di.hDC,r.left,r.top,4,4,_hdc,0,yoffs,SRCCOPY); // top left
      StretchBlt(di.hDC,r.left+4,r.top,r.right-r.left-4-4,4,_hdc,4,yoffs,47-4-4,4,SRCCOPY); // top center
      BitBlt(di.hDC,r.right-4,r.top,4,4,_hdc,47-4,yoffs,SRCCOPY); // top right

      StretchBlt(di.hDC,r.left,r.top+4,4,r.bottom-r.top-4-4,_hdc,0,4+yoffs,4,15-4-4,SRCCOPY); // left edge
      StretchBlt(di.hDC,r.right-4,r.top+4,4,r.bottom-r.top-4-4,_hdc,47-4,4+yoffs,4,15-4-4,SRCCOPY); // right edge

      // center
      StretchBlt(di.hDC,r.left+4,r.top+4,r.right-r.left-4-4,r.bottom-r.top-4-4,_hdc,4,4+yoffs,47-4-4,15-4-4,SRCCOPY);


      BitBlt(di.hDC,r.left,r.bottom-4,4,4,_hdc,0,15-4+yoffs,SRCCOPY); // bottom left
      StretchBlt(di.hDC,r.left+4,r.bottom-4,r.right-r.left-4-4,4,_hdc,4,15-4+yoffs,47-4-4,4,SRCCOPY); // bottom center
      BitBlt(di.hDC,r.right-4,r.bottom-4,4,4,_hdc,47-4,15-4+yoffs,SRCCOPY); // bottom right


			// draw text
      SetBkMode(di.hDC,TRANSPARENT);
			SetTextColor(di.hDC,wadlg_colors[Ord(WADLG_BUTTONFG)]);
      if (di.itemState and ODS_SELECTED) > 0 then
      begin
        Inc(r.left, 2);
        Inc(r.top, 2);
      end;
			DrawText(di.hDC,wt,-1,r,DT_VCENTER or DT_SINGLELINE or DT_CENTER);
      DeleteDC(_hdc);

      if GetFocus = di.hwndItem then
      begin
        pen := CreatePen(PS_SOLID,0,RGB(0,0,0));
        SelectObject(di.hDC,pen);
        WADlg_dotLine(di.hDC,r.left+2,r.top+2,r.right-r.left-3,False);
        WADlg_dotLine(di.hDC,r.right-3,r.top+2,r.bottom-r.top-3,True);
        WADlg_dotLine(di.hDC,r.left+2,r.top+2,r.bottom-r.top-3,True);
        WADlg_dotLine(di.hDC,r.left+2,r.bottom-3,r.right-r.left-3,False);
        DeleteObject(pen);
      end;
		end;
	end;
  case uMsg of
    WM_CTLCOLORLISTBOX,
    WM_CTLCOLORDLG,
    WM_CTLCOLORBTN,
    WM_CTLCOLORSTATIC,
    WM_CTLCOLOREDIT:
    begin
      (* [Saivert; original C code] int bgcolor=(uMsg == WM_CTLCOLOREDIT || uMsg == WM_CTLCOLORLISTBOX) ? wadlg_colors[WADLG_ITEMBG] : (uMsg == WM_CTLCOLORBTN ? wadlg_colors[WADLG_ITEMBG] : wadlg_colors[WADLG_WNDBG]); *)
      if (uMsg = WM_CTLCOLOREDIT) or (uMsg = WM_CTLCOLORLISTBOX) then
        bgcolor := wadlg_colors[Ord(WADLG_ITEMBG)]
      else begin
        if uMsg = WM_CTLCOLORBTN then
          bgcolor := wadlg_colors[Ord(WADLG_ITEMBG)]
        else bgcolor := wadlg_colors[Ord(WADLG_WNDBG)];
      end;

      lb.lbStyle := BS_SOLID;
      lb.lbColor := GetNearestColor(HDC(wParam),bgcolor);

      if wadlg_lastbrush > 0 then DeleteObject(wadlg_lastbrush);
      wadlg_lastbrush := CreateBrushIndirect(lb);
      if uMsg = WM_CTLCOLORSTATIC then
        SetTextColor(HDC(wParam), wadlg_colors[Ord(WADLG_WNDFG)])
      else SetTextColor(HDC(wParam), wadlg_colors[Ord(WADLG_ITEMFG)]);

      SetBkColor(HDC(wParam),lb.lbColor);
      result := Integer(wadlg_lastbrush);
      exit;
    end;
  end;
  result := 0;
end;

function RectInRect(var rect1: TRect; var rect2: TRect): Boolean;
begin
  { this has a bias towards true }

  { this could probably be optimized a lot }
  result := (((rect1.top >= rect2.top) and (rect1.top <= rect2.bottom)) or
      ((rect1.bottom >= rect2.top) and (rect1.bottom <= rect2.bottom)) or
      ((rect2.top >= rect1.top) and (rect2.top <= rect1.bottom)) or
      ((rect2.bottom >= rect1.top) and (rect2.bottom <= rect1.bottom))) { vertical intersect }
      and
      (((rect1.left >= rect2.left) and (rect1.left <= rect2.right)) or
      ((rect1.right >= rect2.left) and (rect1.right <= rect2.right)) or
      ((rect2.left >= rect1.left) and (rect2.left <= rect1.right)) or
      ((rect2.right >= rect1.left) and (rect2.right <= rect1.right))) { horizontal intersect }
      ;
end;

procedure WADlg_removeFromRgn(_hrgn: HRGN; left, top, right, bottom: Integer);
var
  rgn2: HRGN;
begin
  rgn2 := CreateRectRgn(left,top,right,bottom);
  CombineRgn(_hrgn,_hrgn,rgn2,RGN_DIFF);
  DeleteObject(rgn2);
end;

{ [Saivert] original C code "void WADlg_DrawChildWindowBorders(HWND hwndDlg, int *tab, int tabsize)" }
procedure WADlg_DrawChildWindowBorders(hwndDlg: HWND; tab: PInteger; tabsize: Integer);
var
  ps: TPaintStruct;
  _hrgn: HRGN;
  r: TRect;
  pen: HPEN;
  b: HBRUSH;
  o: HGDIOBJ;
  a: Integer;
  left, top: Integer;
begin
  BeginPaint(hwndDlg,ps);

  _hrgn := 0;
  if ps.fErase then
  begin
    r := ps.rcPaint;
    _hrgn := CreateRectRgn(r.left,r.top,r.right,r.bottom);
  end;

  pen := CreatePen(PS_SOLID,0,wadlg_colors[Ord(WADLG_HILITE)]);
  o := SelectObject(ps.hdc,pen);

  while tabsize > 0 do
  begin
    Inc(tab);
    a := tab^;
    Dec(tabsize);
    GetWindowRect(GetDlgItem(hwndDlg,a and $FFFF),r);
    ScreenToClient(hwndDlg,r.TopLeft);
    ScreenToClient(hwndDlg,r.BottomRight);

    if RectInRect(ps.rcPaint,r) then
    begin
      if ((a and $ffff0000) = DCW_SUNKENBORDER) then
      begin
        MoveToEx(ps.hdc,r.left,r.bottom,nil);
        LineTo(ps.hdc,r.right,r.bottom);
        LineTo(ps.hdc,r.right,r.top-1);
        if _hrgn > 0 then
        begin
          WADlg_removeFromRgn(_hrgn,r.left,r.bottom,r.right,r.bottom+1);
          WADlg_removeFromRgn(_hrgn,r.right,r.top,r.right+1,r.bottom);
        end;
      end
      else if ((a and $ffff0000) = DCW_DIVIDER) then
      begin
        if (r.right - r.left < r.bottom - r.top) then // vertical
        begin
          left := (r.left+r.right) div 2;
          MoveToEx(ps.hdc,left,r.top,nil);
          LineTo(ps.hdc,left,r.bottom+1);
          if _hrgn > 0 then WADlg_removeFromRgn(_hrgn,left,r.top,left+1,r.bottom);
        end
        else // horiz
        begin
          top := (r.top+r.bottom) div 2;
          MoveToEx(ps.hdc,r.left,top,nil);
          LineTo(ps.hdc,r.right+1,top);
          if _hrgn > 0 then WADlg_removeFromRgn(_hrgn,r.left,top,r.right,top+1);
        end;
      end;
    end;
  end;

  SelectObject(ps.hdc,o);
  DeleteObject(pen);

  if _hrgn > 0 then
  begin
    //erase bkgnd while clipping out our own drawn stuff (for flickerless display)
    b := CreateSolidBrush(wadlg_colors[Ord(WADLG_WNDBG)]);
    FillRgn(ps.hdc,_hrgn,b);
    DeleteObject(b);
    DeleteObject(_hrgn);
  end;
  EndPaint(hwndDlg,ps);
end;
{$IFEND}

initialization
  wadlg_defcolors[0] := RGB(0,0,0);
  wadlg_defcolors[1] := RGB(0,255,0);
  wadlg_defcolors[2] := RGB(36,36,60);
  wadlg_defcolors[3] := RGB(57,56,66);
  wadlg_defcolors[4] := RGB(255,255,255);
  wadlg_defcolors[5] := RGB(132,148,165);
  wadlg_defcolors[6] := RGB(0,0,198);
  wadlg_defcolors[7] := RGB(36*2,36*2,60*2);
  wadlg_defcolors[8] := RGB(255,255,255);
  wadlg_defcolors[9] := RGB(36*3,36*3,60*3);
  wadlg_defcolors[10] := RGB(36,36,60);
  wadlg_defcolors[11] := RGB(Round(36*0.5),Round(36*0.5),Round(60*0.5));
  wadlg_defcolors[12] := RGB(36,36,60);
  wadlg_defcolors[13] := RGB(36*1,36*1,60*1);
  wadlg_defcolors[14] := RGB(36*1,36*1,60*1);
  wadlg_defcolors[15] := RGB(121,130,150);
  wadlg_defcolors[16] := RGB(78,88,110);
  wadlg_defcolors[17] := RGB(36*1,36*1,60*1)

end.

